searchData={"items":[{"type":"module","title":"EctoAnon","doc":"`EctoAnon` provides a simple way to handle data anonymization directly in your Ecto schemas.","ref":"EctoAnon.html"},{"type":"function","title":"EctoAnon.run/3","doc":"Updates an Ecto struct with anonymized data based on its anon_schema declaration.\nIt also updates any embedded schema declared in anon_schema.\n\n    defmodule User do\n      use Ecto.Schema\n      use EctoAnon.Schema\n\n      anon_schema [\n        :firstname,\n        email: &__MODULE__.anonymized_email/3\n        birthdate: [:anonymized_date, options: [:only_year]]\n      ]\n\n      schema \"user\" do\n        field :fistname, :string\n        field :email, :string\n        field :birthdate, :utc_datetime\n      end\n\n      def anonymized_email(_type, _value, _opts) do\n        \"xxx@xxx.com\"\n      end\n    end\n\nIt returns `{:ok, struct}` if the struct has been successfully updated or `{:error, :non_anonymizable_struct}` if the struct has no anonymizable fields.","ref":"EctoAnon.html#run/3"},{"type":"function","title":"Options - EctoAnon.run/3","doc":"* `:cascade` - When set to `true`, allows ecto_anon to preload and anonymize\n  all associations (and associations of these associations) automatically in cascade.\n  Could be used to anonymize all data related a struct in a single call.\n  Note that this won't traverse `belongs_to` associations to avoid infinite and cyclic anonymizations.\n  Default: false\n\n  * `:log`- When set to `true`, it will set `anonymized` field when EctoAnon.run\n  applies anonymization on a ressource.\n  Default: true","ref":"EctoAnon.html#run/3-options"},{"type":"function","title":"Example - EctoAnon.run/3","doc":"defmodule User do\n      use Ecto.Schema\n      use EctoAnon.Schema\n\n      anon_schema [\n        :email\n      ]\n\n      schema \"users\" do\n        field :name, :string\n        field :age, :integer, default: 0\n        field :email, :string\n      end\n    end\n\nBy default, an anon_field will be anonymized with a default value based on its type:\n\n    iex> user = Repo.get(User, id)\n    %User{name: \"jane\", age: 0, email: \"jane@email.com\"}\n\n    iex> EctoAnon.run(user, Repo)\n    {:ok, %User{name: \"jane\", age: 0, email: \"redacted\"}}","ref":"EctoAnon.html#run/3-example"},{"type":"module","title":"EctoAnon.Anonymizer","doc":"","ref":"EctoAnon.Anonymizer.html"},{"type":"function","title":"EctoAnon.Anonymizer.anonymized_data/1","doc":"","ref":"EctoAnon.Anonymizer.html#anonymized_data/1"},{"type":"function","title":"EctoAnon.Anonymizer.is_association?/2","doc":"","ref":"EctoAnon.Anonymizer.html#is_association?/2"},{"type":"function","title":"EctoAnon.Anonymizer.is_embed?/2","doc":"","ref":"EctoAnon.Anonymizer.html#is_embed?/2"},{"type":"module","title":"EctoAnon.Migration","doc":"","ref":"EctoAnon.Migration.html"},{"type":"function","title":"EctoAnon.Migration.anonymized/0","doc":"Adds anonymized column (boolean) to a table. This column can be used to track if the resource has be anonymized or not.\n\n    defmodule MyApp.Repo.Migrations.CreateUser do\n      use Ecto.Migration\n      import EctoAnon.Migration\n\n      def change do\n        create table(:users) do\n          add :firstname, :string\n          add :lastname, :string\n          timestamps()\n          anonymized()\n        end\n      end\n    end","ref":"EctoAnon.Migration.html#anonymized/0"},{"type":"module","title":"EctoAnon.Query","doc":"","ref":"EctoAnon.Query.html"},{"type":"function","title":"EctoAnon.Query.apply/3","doc":"","ref":"EctoAnon.Query.html#apply/3"},{"type":"function","title":"EctoAnon.Query.not_anonymized/1","doc":"Returns a query that searches resources excluding anonymized ones\n\n    query = from(u in User, select: u)\n    |> not_anonymized\n\n    results = Repo.all(query)","ref":"EctoAnon.Query.html#not_anonymized/1"},{"type":"function","title":"EctoAnon.Query.set_anonymized/2","doc":"","ref":"EctoAnon.Query.html#set_anonymized/2"},{"type":"module","title":"EctoAnon.Schema","doc":"EctoAnon.Schema lets you define anonymizable fields.","ref":"EctoAnon.Schema.html"},{"type":"macro","title":"EctoAnon.Schema.anon_schema/1","doc":"Macro to declare anonymizable fields","ref":"EctoAnon.Schema.html#anon_schema/1"},{"type":"macro","title":"EctoAnon.Schema.anonymized/0","doc":"Macro to create an anonymized field in the schema","ref":"EctoAnon.Schema.html#anonymized/0"},{"type":"behaviour","title":"EctoAnon.Functions","doc":"This module is in charge of the function attribution.","ref":"EctoAnon.Functions.html"},{"type":"function","title":"EctoAnon.Functions.get_function/1","doc":"Returns the anonymization function based on passed argument","ref":"EctoAnon.Functions.html#get_function/1"},{"type":"function","title":"EctoAnon.Functions.get_module/1","doc":"Returns the anonymization module based on passed argument","ref":"EctoAnon.Functions.html#get_module/1"},{"type":"callback","title":"EctoAnon.Functions.run/3","doc":"","ref":"EctoAnon.Functions.html#c:run/3"},{"type":"module","title":"EctoAnon.Functions.AnonymizedDate","doc":"Anonymizing module for a date, accepting types:\n  - date\n  - utc_datetime\n  - utc_datetime_usec\n  - naive_datetime\n  - naive_datetime_usec","ref":"EctoAnon.Functions.AnonymizedDate.html"},{"type":"function","title":"EctoAnon.Functions.AnonymizedDate.run/3","doc":"Returns an anonymized date/datetime","ref":"EctoAnon.Functions.AnonymizedDate.html#run/3"},{"type":"function","title":"Options - EctoAnon.Functions.AnonymizedDate.run/3","doc":"* `:only_year` - When set to `true`, it keeps the year while anonymizing the rest","ref":"EctoAnon.Functions.AnonymizedDate.html#run/3-options"},{"type":"function","title":"Example - EctoAnon.Functions.AnonymizedDate.run/3","doc":"{:ok, date} = Date.new(2019, 02, 04)\n\n    AnonymizedDate.run(:date, date, [:only_year])\n    ~D[2019-01-01]\n\n    AnonymizedDate.run(:utc_datetime, date, [:only_year])\n    ~U[2019-01-01 00:00:00Z]\n\n    AnonymizedDate.run(:utc_datetime_usec, date, [:only_year])\n    ~U[2019-01-01 00:00:00.000000Z]\n\n    AnonymizedDate.run(:naive_datetime, date, [:only_year])\n    ~N[2019-01-01 00:00:00]\n\n    AnonymizedDate.run(:naive_datetime_usec, date, [:only_year])\n    ~N[2019-01-01 00:00:00.000000]\n\n    AnonymizedDate.run(:date, date)\n    ~D[1970-01-01]","ref":"EctoAnon.Functions.AnonymizedDate.html#run/3-example"},{"type":"module","title":"EctoAnon.Functions.AnonymizedEmail","doc":"Anonymizing module for an email","ref":"EctoAnon.Functions.AnonymizedEmail.html"},{"type":"function","title":"EctoAnon.Functions.AnonymizedEmail.run/3","doc":"Returns an anonymized email with hashes while keeping the email format","ref":"EctoAnon.Functions.AnonymizedEmail.html#run/3"},{"type":"function","title":"Options - EctoAnon.Functions.AnonymizedEmail.run/3","doc":"* `:partial` - When set to `true`, it keeps the first character of each part and hides the rest","ref":"EctoAnon.Functions.AnonymizedEmail.html#run/3-options"},{"type":"function","title":"Example - EctoAnon.Functions.AnonymizedEmail.run/3","doc":"AnonymizedEmail.run(:string, \"john.doe@email.com\", [])\n    \"5d526ea414@a717679b0f.com\"\n\n    AnonymizedEmail.run(:string, \"not_an_email\", [])\n    \"c2fd076ad9@c2fd076ad9.com\"\n\n    AnonymizedEmail.run(:string, \"john.doe@email.com\", partial: true)\n    \"j******@e******.com\"","ref":"EctoAnon.Functions.AnonymizedEmail.html#run/3-example"},{"type":"module","title":"EctoAnon.Functions.AnonymizedPhone","doc":"Anonymizing module for a phone number\n\nCurrently only returns with french format","ref":"EctoAnon.Functions.AnonymizedPhone.html"},{"type":"function","title":"EctoAnon.Functions.AnonymizedPhone.run/3","doc":"Returns a phone number anonymized with french format","ref":"EctoAnon.Functions.AnonymizedPhone.html#run/3"},{"type":"function","title":"Example - EctoAnon.Functions.AnonymizedPhone.run/3","doc":"AnonymizedPhone.run(:string, \"0601020304\", [])\n    \"xx xx xx xx xx\"","ref":"EctoAnon.Functions.AnonymizedPhone.html#run/3-example"},{"type":"module","title":"EctoAnon.Functions.Default","doc":"Default anonymizing functions for basic Ecto types.","ref":"EctoAnon.Functions.Default.html"},{"type":"function","title":"EctoAnon.Functions.Default.run/3","doc":"Returns default anonymized value based on field type\n\n    @default_integer 0\n    @default_float 0.0\n    @default_string \"redacted\"\n    @default_map %{}\n    @default_decimal Decimal.new(\"0.0\")\n    @default_date ~D[1970-01-01]\n    @default_datetime ~U[1970-01-01 00:00:00Z]\n    @default_datetime_usec ~U[1970-01-01 00:00:00.000000Z]\n    @default_naive_datetime ~N[1970-01-01 00:00:00]\n    @default_naive_datetime_usec ~N[1970-01-01 00:00:00.000000]\n    @default_time ~T[00:00:00]\n    @default_time_usec ~T[00:00:00.000000]","ref":"EctoAnon.Functions.Default.html#run/3"},{"type":"module","title":"EctoAnon.Functions.Helpers","doc":"","ref":"EctoAnon.Functions.Helpers.html"},{"type":"function","title":"EctoAnon.Functions.Helpers.generate_hash/1","doc":"","ref":"EctoAnon.Functions.Helpers.html#generate_hash/1"},{"type":"module","title":"EctoAnon.Functions.RandomUuid","doc":"Genererates a random uuid version 4","ref":"EctoAnon.Functions.RandomUuid.html"},{"type":"function","title":"EctoAnon.Functions.RandomUuid.run/3","doc":"Returns a random version 4 UUID","ref":"EctoAnon.Functions.RandomUuid.html#run/3"},{"type":"function","title":"Example - EctoAnon.Functions.RandomUuid.run/3","doc":"RandomUuid.run(:string, \"0601020304\", [])\n    \"71d8ed69-31ca-4b9d-826b-f355ea7fb128\"","ref":"EctoAnon.Functions.RandomUuid.html#run/3-example"},{"type":"extras","title":"Overview","doc":"# ecto_anon\n\n[![Module Version](https://img.shields.io/hexpm/v/ecto_anon.svg)](https://hex.pm/packages/ecto_anon)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ecto_anon/)\n[![Total Download](https://img.shields.io/hexpm/dt/ecto_anon.svg)](https://hex.pm/packages/ecto_anon)\n[![License](https://img.shields.io/hexpm/l/ecto_anon)](https://github.com/WTTJ/ecto_anon/blob/main/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/WTTJ/ecto_anon.svg)](https://github.com/WTTJ/ecto_anon/commits/main)\n\nSimple way to handle data anonymization directly in your [Ecto](https://github.com/elixir-ecto/ecto) schemas\n\n---","ref":"readme.html"},{"type":"extras","title":"Table of Contents - Overview","doc":"- [Installation](#installation)\n- [Usage](#usage)\n  - [Options](#options)\n  - [Default values](#default-values)\n  - [Native functions](#native-functions)\n  - [Custom functions](#custom-functions)\n  - [Migrations](#migrations)\n  - [Filtering](#filtering)\n- [License](#copyright-and-license)\n\n# Installation\n\nAdd `:ecto_anon` to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:ecto_anon, \"~> 0.6.0\"}\n  ]\nend\n```\n\n# Usage\n\nDefine an `anon_schema` in your schema module and specify every fields you want to anonymize (regular fields, associations, embeds):\n\n```elixir\ndefmodule User do\n  use Ecto.Schema\n  use EctoAnon.Schema\n\n  anon_schema [\n    :name,\n    :email\n  ]\n\n  schema \"users\" do\n    field :name, :string\n    field :age, :integer\n    field :email, :string\n\n    anonymized()\n  end\nend\n```\n\nThen use `EctoAnon.run` to apply anonymization on desired resource\n\n```elixir\nuser = Repo.get(User, id)\n%User{name: \"jane\", age: 24, email: \"jane@email.com\"}\n\nEctoAnon.run(user, Repo)\n{:ok, %User{name: \"redacted\", age: 24, email: \"redacted\"}}\n```","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Options - Overview","doc":"### `cascade`\n\nWhen set to `true`, it allows `ecto_anon` to preload and anonymize\nall associations (and associations of these associations) automatically in cascade.\nCould be used to anonymize all data related to a struct in a single call.\n\nNote that this won't traverse `belongs_to` associations.\n\nDefault: `false`\n\n```elixir\ndefmodule User do\n  use Ecto.Schema\n  use EctoAnon.Schema\n\n  anon_schema [\n    :lastname,\n    :email,\n    :followers,\n    :favorite_quote,\n    :quotes,\n    last_sign_in_at: [:anonymized_date, options: [:only_year]]\n  ]\n\n  schema \"users\" do\n    field(:firstname, :string)\n    field(:lastname, :string)\n    field(:email, :string)\n    field(:last_sign_in_at, :utc_datetime)\n\n    has_many(:comments, Comment, foreign_key: :author_id, references: :id)\n    embeds_one(:favorite_quote, Quote)\n    embeds_many(:quotes, Quote)\n\n    many_to_many(\n      :followers,\n      __MODULE__,\n      join_through: Follower,\n      join_keys: [follower_id: :id, followee_id: :id]\n    )\n\n    anonymized()\n  end\nend\n\ndefmodule Quote do\n  use Ecto.Schema\n  use EctoAnon.Schema\n\n  anon_schema([\n    :quote,\n    :author\n  ])\n\n  embedded_schema do\n    field(:quote, :string)\n    field(:author, :string)\n  end\nend\n\ndefmodule Follower do\n  use Ecto.Schema\n\n  schema \"followers\" do\n    field(:follower_id, :id)\n    field(:followee_id, :id)\n    timestamps()\n  end\nend\n```\n\n```elixir\nRepo.get(User, id)\n|> EctoAnon.run(Repo, cascade: true)\n\n{:ok,\n   %User{\n     email: \"redacted\",\n     firstname: \"John\",\n     last_sign_in_at: ~U[2022-01-01 00:00:00Z],\n     lastname: \"redacted\",\n     favorite_quote: %Quote{\n       quote: \"redacted\",\n       author: \"redacted\"\n     },\n     quotes: [\n       %Quote{\n         quote: \"redacted\",\n         author: \"redacted\"\n       },\n       %Quote{\n         quote: \"redacted\",\n         author: \"redacted\"\n       }\n     ]\n   }\n }\n```\n\n### `log`\n\nWhen set to `true`, it will set `anonymized` field accordingly when `EctoAnon.run` is called on a ressource.\n\nDefault: `true`","ref":"readme.html#options"},{"type":"extras","title":"Default values - Overview","doc":"By default, a field will be anonymized with those valuee, based on its type:\n\n| type                | value                             |\n| ------------------- | --------------------------------- |\n| integer             | 0                                 |\n| float               | 0.0                               |\n| string              | redacted                          |\n| map                 | %{}                               |\n| decimal             | Decimal . new ( \" 0.0 \" )         |\n| date                | ~D[ 1970-01-01 ]                  |\n| datetime            | ~U[ 1970-01-01 00:00:00Z ]        |\n| datetime_usec       | ~U[ 1970-01-01 00:00:00.000000Z ] |\n| naive_datetime      | ~N[ 1970-01-01 00:00:00 ]         |\n| naive_datetime_usec | ~N[ 1970-01-01 00:00:00.000000 ]  |\n| time                | ~T[ 00:00:00 ]                    |\n| time_usec           | ~T[ 00:00:00.000000 ]             |\n| boolean             | no change                         |\n| id                  | no change                         |\n| binary_id           | no change                         |\n| binary              | no change                         |","ref":"readme.html#default-values"},{"type":"extras","title":"Native functions - Overview","doc":"```elixir\nanon_schema([\n  email: :anonymized_email,\n  birthdate: [:anonymized_date, options: [:only_year]]\n])\n```\n\nNatively, `ecto_anon` embeds differents functions to suit your needs\n\n| function          | role                                               | options    |\n| ----------------- | -------------------------------------------------- | ---------- |\n| :anonymized_date  | Anonymizes partially or completely a date/datetime | :only_year |\n| :anonymized_email | Anonymizes partially or completely an email        | :partial   |\n| :anonymized_phone | Anonymizes a phone number (currently only FR)      |            |\n| :random_uuid      | Returns a random UUID                              |            |","ref":"readme.html#native-functions"},{"type":"extras","title":"Custom functions - Overview","doc":"```elixir\nanon_schema([\n    address: &__MODULE__.anonymized_address/3\n])\n\ndef anonymized_address(:map, %{} = address, _opts \\\\ []) do\n  address\n  |> Map.drop([\"street\"])\nend\n```\n\nYou can also pass custom functions with the following signature: `function(type, value, options)`","ref":"readme.html#custom-functions"},{"type":"extras","title":"Migrations - Overview","doc":"By importing `EctoAnon.Migration` in your ecto migration file, you can add an `anonymized()` macro that will generate an `anonymized` boolean field in your table:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.CreateUser do\n  use Ecto.Migration\n  import EctoAnon.Migration\n\n  def change do\n    create table(:users) do\n      add :firstname, :string\n      add :lastname, :string\n      timestamps()\n      anonymized()\n    end\n  end\nend\n```\n\nCombined with `log` option when executing the anonymization, it will allow you to identify anonymized rows and exclude them in your queries with `EctoAnon.Query.not_anonymized/1`.","ref":"readme.html#migrations"},{"type":"extras","title":"Filtering - Overview","doc":"As you can create an [anonymized field in your migration](#migrations), you can add `anonymized()` in your schema, just like `timestamps()`.\n\nBy adding this field, you can use it to filter your resources and exclude anonymized data easily:\n\n```elixir\nimport EctoAnon.Query\nimport Ecto.Query\n\nfrom(u in User, select: u)\n|> not_anonymized()\n|> Repo.all()\n```\n\n# Copyright and License\n\n_Copyright (c) 2022 CORUSCANT (Welcome to the Jungle) - https://www.welcometothejungle.com_\n\n_This library is licensed under the [MIT](LICENSE.md)_","ref":"readme.html#filtering"},{"type":"extras","title":"License","doc":"The MIT License\n\nCopyright (c) 2022 CORUSCANT (Welcome to the Jungle) - https://www.welcometothejungle.com\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}